
#include <iostream>

#include <Graph.h>
#include <FreqGeo.h>

static void
testExtend (void)
{
	std::vector<double> pos (3);

	FriGeo::Graph g1;
	pos[0] = 0; pos[1] = 0; pos[2] = 0; g1.PushVertex (pos, 1);
	pos[0] = 1; pos[1] = 0; pos[2] = 0; g1.PushVertex (pos, 2);
	pos[0] = 0; pos[1] = 1; pos[2] = 0; g1.PushVertex (pos, 3);
	pos[0] = 1; pos[1] = 1; pos[2] = 0; g1.PushVertex (pos, 3);
	g1.PushEdge (0, 1, 0);
	g1.PushEdge (1, 2, 0);
	g1.PushEdge (1, 3, 0);
	g1.Seal ();

	FriGeo::Graph gsub;
	pos[0] = 1; pos[1] = 1; pos[2] = 1; gsub.PushVertex (pos, 1);
	pos[0] = 2; pos[1] = 1; pos[2] = 1; gsub.PushVertex (pos, 2);
	gsub.PushEdge (0, 1, 0);
	gsub.Seal ();

	FriGeo::Graph gsubQ;
	pos[0] = 1; pos[1] = 2; pos[2] = 1; gsubQ.PushVertex (pos, 3);
	gsubQ.Seal ();

	std::vector<unsigned int> Qedge (3);
	Qedge[0] = 1; Qedge[1] = 2; Qedge[2] = 0;

	std::vector<unsigned int> PI (2);
	PI[0] = 0; PI[1] = 1;

	std::vector<std::vector<unsigned int> > matches;
	g1.Extend (gsub, PI, gsubQ, Qedge, matches);
}

static void
testBridge (void)
{
	std::vector<double> pos (3);

	FriGeo::Graph g1;
	g1.PushVertex (pos, 1);
	g1.PushVertex (pos, 1);
	g1.PushVertex (pos, 1);
	g1.PushVertex (pos, 1);
	g1.PushVertex (pos, 1);
	g1.PushVertex (pos, 1);

	g1.PushEdge (0, 1, 0);
	g1.PushEdge (1, 2, 0);
	g1.PushEdge (2, 0, 0);
	g1.PushEdge (2, 3, 0);
	g1.PushEdge (3, 4, 0);
	g1.PushEdge (4, 5, 0);
	g1.PushEdge (5, 3, 0);
	g1.Seal ();

	std::vector<unsigned int> bridges;
	g1.ListBridges (bridges);


	FriGeo::Graph g2;
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 1);

	g2.PushEdge (0, 1, 0);
	g2.PushEdge (1, 2, 0);
	g2.PushEdge (2, 0, 0);
	g2.PushEdge (2, 3, 0);
	g2.PushEdge (3, 4, 0);
	g2.PushEdge (4, 2, 0);
	g2.PushEdge (4, 5, 0);
	g2.PushEdge (2, 6, 0);
	g2.PushEdge (6, 7, 0);
	g2.Seal ();

	std::vector<unsigned int> bridges2;
	g2.ListBridges (bridges2);
}

static void
testListExtensions (void)
{
	std::vector<double> pos (3);

	FriGeo::Graph g2;
	g2.PushVertex (pos, 0);
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 2);
	g2.PushVertex (pos, 3);
	g2.PushVertex (pos, 4);
	g2.PushVertex (pos, 5);
	g2.PushVertex (pos, 6);
	g2.PushVertex (pos, 7);

	g2.PushEdge (0, 1, 0);
	g2.PushEdge (1, 2, 0);
	g2.PushEdge (2, 0, 0);
	g2.PushEdge (2, 3, 0);
	g2.PushEdge (3, 4, 0);
	g2.PushEdge (4, 2, 0);
	g2.PushEdge (4, 5, 0);
	g2.PushEdge (2, 6, 0);
	g2.PushEdge (6, 7, 0);
	g2.Seal ();

	/* Empty graph
	 */
	FriGeo::Graph P;
	P.Seal ();
	std::vector<unsigned int> PI;

	do {
		std::vector<FriGeo::Graph> extensionsEdge;
		std::vector<std::pair<FriGeo::Graph, std::vector<unsigned int> > > extensionsEdgeVertex;
		g2.ListExtensions (P, PI, extensionsEdge, extensionsEdgeVertex);

		std::cout << extensionsEdge.size () << " edge extensions, "
			<< extensionsEdgeVertex.size () << " edge-vertex extensions." << std::endl;

		if (extensionsEdgeVertex.size () == 0)
			break;

		P = extensionsEdgeVertex[0].first;
		PI = extensionsEdgeVertex[0].second;
		std::cout << "first: " << P << std::endl;
	} while (1);
}


static void
testMine (void)
{
	std::vector<double> pos (3);

	FriGeo::Graph g2;
	g2.PushVertex (pos, 0);
	g2.PushVertex (pos, 1);
	g2.PushVertex (pos, 2);
	g2.PushVertex (pos, 3);
	g2.PushVertex (pos, 4);
	g2.PushVertex (pos, 5);
	g2.PushVertex (pos, 6);
	g2.PushVertex (pos, 7);

	g2.PushEdge (0, 1, 0);
	g2.PushEdge (1, 2, 0);
	g2.PushEdge (2, 0, 0);
	g2.PushEdge (2, 3, 0);
	g2.PushEdge (3, 4, 0);
	g2.PushEdge (4, 2, 0);
	g2.PushEdge (4, 5, 0);
	g2.PushEdge (2, 6, 0);
	g2.PushEdge (6, 7, 0);
	g2.Seal ();

	std::vector<FriGeo::Graph> G;
	G.push_back (g2);

	FriGeo::FreqGeo fg (G, g2, 1);
	fg.Mine ();
}

static void
testMine2 (void)
{
	std::vector<double> pos (3);

	FriGeo::Graph g2;
	pos[0] = 0; pos[1] = 0; pos[2] = 0; g2.PushVertex (pos, 0);
	pos[0] = 0; pos[1] = 1; pos[2] = 0; g2.PushVertex (pos, 1);
	pos[0] = 0; pos[1] = 0; pos[2] = 1; g2.PushVertex (pos, 2);
	g2.PushEdge (0, 1, 0);
	g2.PushEdge (1, 2, 1);
	g2.PushEdge (2, 0, 2);
	g2.Seal ();

	std::vector<FriGeo::Graph> G;
	G.push_back (g2);

	FriGeo::FreqGeo fg (G, g2, 1);
	fg.Mine ();
}

static void
testMine3 (void)
{
	std::vector<double> pos (3);

	FriGeo::Graph g2;
	pos[0] = 0; pos[1] = 0; pos[2] = 0; g2.PushVertex (pos, 0);
	pos[0] = 0; pos[1] = 0; pos[2] = 1; g2.PushVertex (pos, 1);
	pos[0] = 0; pos[1] = 1; pos[2] = 1; g2.PushVertex (pos, 0);
	g2.PushEdge (0, 1, 0);
	g2.PushEdge (1, 2, 0);
	g2.Seal ();

	FriGeo::Graph g3;
	pos[0] = 0; pos[1] = 0; pos[2] = 0; g3.PushVertex (pos, 0);
	pos[0] = 0; pos[1] = 0; pos[2] = 1; g3.PushVertex (pos, 1);
	pos[0] = 0; pos[1] = 1; pos[2] = 1; g3.PushVertex (pos, 2);
	g3.PushEdge (0, 1, 0);
	g3.PushEdge (1, 2, 0);
	g3.Seal ();

	std::vector<FriGeo::Graph> G;
	G.push_back (g2);
	G.push_back (g3);

	FriGeo::FreqGeo fg (G, g2, 2);
	fg.Mine ();
}

int
main (int argc, char* argv[])
{
	FriGeo::Graph g;

	std::vector<double> pos (3);
	pos[0] = 0; pos[1] = 1; pos[2] = 2;
	g.PushVertex (pos, 14);
	pos[0] = 4; pos[1] = 2; pos[2] = 4;
	g.PushVertex (pos, 7);
	pos[0] = 3; pos[1] = 0; pos[2] = 3;
	g.PushVertex (pos, 4);

	g.PushEdge (0, 1, 11);
	g.PushEdge (0, 2, 10);
	g.Seal ();

	std::cout << g;

	testExtend ();
	testBridge ();
	testListExtensions ();
	testMine ();
	testMine2 ();
	testMine3 ();
}


