function [ROC_frigeo, ROC_gspan] = exp3_roceval(files);

% exp3_roceval(dir('bla*.mat'));

mest_m = 1;
mest_p = 0.5;

ROC_frigeo=zeros(0,4);
ROC_gspan=zeros(0,4);
for i=1:length(files)
	C=load(files(i).name);
	G_train = C.G_train;
	G_test = C.G_test;
	[train_auc, train_eer, train_curve, test_auc, test_eer, test_curve] = ...
		compute_roc(C.res_frigeo.occ, C.idx, G_train, G_test, mest_m, mest_p);
	ROC_frigeo=[ROC_frigeo; train_auc, train_eer, test_auc, test_eer];

	[train_auc, train_eer, train_curve, test_auc, test_eer, test_curve] = ...
		compute_roc(C.res_gspan.gspan_occ, C.idx, G_train, G_test, mest_m, mest_p);
	ROC_gspan=[ROC_gspan; train_auc, train_eer, test_auc, test_eer];
end


function [train_auc, train_eer, train_curve, test_auc, test_eer, test_curve] = ...
	compute_roc(occ, idx, G_train, G_test, mest_m, mest_p);

% mest_m is the equivalent sample size for the m-estimate of p(F_i | C_j),
% mest_p is the prior probability.

% Compute ROC statistics by naive bayes.
Ytrain = get_y(G_train);
Ytest = get_y(G_test);
Xtrain=double(occ(1:length(G_train),:) > 0);
Xtest=double(occ((length(G_train)+1):end,:) > 0);

% p(F_i == 1 | C == 1), m-estimate
Fpos=(sum(Xtrain(find(Ytrain == 1),:)) + mest_m*mest_p) ./ (sum(Ytrain == 1) + mest_m);
% p(F_i == 1 | C == -1), m-estimate
Fneg=(sum(Xtrain(find(Ytrain == -1),:)) + mest_m*mest_p) ./ (sum(Ytrain == -1) + mest_m);
pos_w=log(Fpos ./ Fneg);

% p(F_i == -1 | C == 1), m-estimate
Fpos=(sum(1 - Xtrain(find(Ytrain == 1),:)) + mest_m*mest_p) ./ (sum(Ytrain == 1) + mest_m);
% p(F_i == -1 | C == -1), m-estimate
Fneg=(sum(1 - Xtrain(find(Ytrain == -1),:)) + mest_m*mest_p) ./ (sum(Ytrain == -1) + mest_m);
neg_w=log(Fpos ./ Fneg);

% p(C == 1)
PCpos=sum(Ytrain == 1) / length(Ytrain);
% p(C == -1)
PCneg=sum(Ytrain == -1) / length(Ytrain);

% Evaluate training/test error
pred_train = predict(pos_w, neg_w, PCpos, PCneg, Xtrain, Ytrain);
pred_test = predict(pos_w, neg_w, PCpos, PCneg, Xtest, Ytest);
[train_auc, train_eer, train_curve] = rocscore(pred_train, Ytrain);
[test_auc, test_eer, test_curve] = rocscore(pred_test, Ytest);

function [pred_pos] = predict(pos_w, neg_w, PCpos, PCneg, X, Y);
N=length(Y);
pred_pos=ones(N,1)*log(PCpos/PCneg);	% class prior
for i=1:N
	for j=1:size(X,2)
		if X(i,j) > 0
			pred_pos(i) = pred_pos(i) + pos_w(j);
		else
			pred_pos(i) = pred_pos(i) + neg_w(j);
		end
	end
end


function y = get_y(G);
y=zeros(length(G),1);
for i=1:length(G)
	y(i) = G{i}.Y;
end

